Unit uSprite;


interface

Uses
   types,Graph,Setting;

const
    REDRAW_ALL = 0;
    REDRAW_TURN = 1;
    REDRAW_GAME = 2;

type typeSPRITE = record
    bHidden:BOOLEAN;
    BoundRect:typeRECTANGLE;
    pBitmap:POINTER;
    sBitmap:WORD;
    num:INTEGER;
    FnBound:function( num:integer; var Match:typeMATCH_STATE; var Bound:typeRECTANGLE):BOOLEAN;
    FnDraw:procedure( num:integer; var Match:typeMATCH_STATE);
end;

var
    FnDrawBack:procedure;

procedure RemoveAllSprites;
procedure AddSprite( var Sprite:typeSPRITE );
procedure AddTurnSprite( var Sprite:typeSPRITE );
procedure HideSprite( var Sprite:typeSPRITE );
procedure ResetSprite( var Sprite:typeSPRITE );
procedure DrawSprite( var Sprite:typeSPRITE; var Match:typeMATCH_STATE );
procedure OverHideSprite( i:integer; var Match:typeMATCH_STATE );
procedure DrawWorld( Redraw:integer; var Match:typeMATCH_STATE);

implementation

const
    MAX_SPRITES = 30;

var
    Sprites:array[1..MAX_SPRITES] of typeSPRITE;
    nSprites:integer;
    TurnSprites:array[1..MAX_SPRITES] of typeSPRITE;
    nTurnSprites:integer;



function max(a,b:integer):integer;
begin
  if (a>b) then
      max := a
  else
      max := b;
end;

function min(a,b:integer):integer;
begin
  if (a<b) then
      min := a
  else
      min := b;
end;

function Intersect( var rc1,rc2:typeRECTANGLE ):BOOLEAN;
begin
    if (max(rc1.x1,rc2.x1) < min (rc1.x2,rc2.x2)) and
       (max(rc1.y1,rc2.y1) < min (rc1.y2,rc2.y2)) then
           Intersect := TRUE
    else
           Intersect := FALSE;
end;

procedure IntersectApply( var rc1,rc2:typeRECTANGLE );
begin
    rc1.x1 := max(rc1.x1,rc2.x1);
    rc1.y1 := max(rc1.y1,rc2.y1);
    rc1.x2 := min(rc1.x2,rc2.x2);
    rc1.y2 := min(rc1.y2,rc2.y2);

    if (rc1.x1 > rc1.x2) or (rc1.y1 > rc1.y2) then
    begin
       rc1.x1 := 0;
       rc1.y1 := 0;
       rc1.x2 := 0;
       rc1.y2 := 0;
    end;
end;

procedure RemoveAllSprites;
begin
    nSprites := 0;
end;

procedure AddSprite( var Sprite:typeSPRITE );
begin
    inc(nSprites);
    Sprites[nSprites] := Sprite;
    Sprites[nSprites].bHidden := TRUE;
end;

procedure AddTurnSprite( var Sprite:typeSPRITE );
begin
    inc(nTurnSprites);
    TurnSprites[nTurnSprites] := Sprite;
    TurnSprites[nTurnSprites].bHidden := TRUE;
end;

procedure HideSprite( var Sprite:typeSPRITE );
begin
    if (Sprite.bHidden <> TRUE) then
    begin
        Sprite.bHidden := TRUE;
        PutImage(Sprite.BoundRect.x1,Sprite.BoundRect.y1, Sprite.pBitmap^, NormalPut);
        {Graph.bar(Sprite.BoundRect.x1,Sprite.BoundRect.y1,Sprite.BoundRect.x2,Sprite.BoundRect.y2);}
        FreeMem(Sprite.pBitmap,Sprite.sBitmap);
    end;
end;

procedure ResetSprite( var Sprite:typeSPRITE );
begin
     if (Sprite.bHidden = FALSE) then
     begin
         Sprite.bHidden := TRUE;
         FreeMem(Sprite.pBitmap,Sprite.sBitmap);
     end;
end;

procedure DrawSprite( var Sprite:typeSPRITE; var Match:typeMATCH_STATE );
begin
    if (Sprite.bHidden = TRUE) then
    begin
         if (Sprite.FnBound(Sprite.num,Match,Sprite.BoundRect)) then
         begin
             {Sprite has to be drawn}
             Sprite.bHidden := FALSE;
             IntersectApply(Sprite.BoundRect,Settings.ScreenRect);
             Sprite.sBitmap := Graph.ImageSize(Sprite.BoundRect.x1,Sprite.BoundRect.y1,
                                               Sprite.BoundRect.x2,Sprite.BoundRect.y2);
             GetMem(Sprite.pBitmap, Sprite.sBitmap);   { Allocate memory on heap }
             GetImage(Sprite.BoundRect.x1,Sprite.BoundRect.y1,Sprite.BoundRect.x2,Sprite.BoundRect.y2, Sprite.pBitmap^);
             Sprite.FnDraw(Sprite.num,Match);
         end;
    end;
end;

procedure OverHideSprite( i:integer; var Match:typeMATCH_STATE );
var j:integer;
var iRect,jRect:typeRECTANGLE;
begin
    if (Sprites[i].bHidden <> TRUE) then
    begin
       Sprites[i].FnBound(Sprites[i].num,Match,iRect);

       for j := i+1 to nSprites do begin
          if (Sprites[j].bHidden <> TRUE) then
          begin
            Sprites[j].FnBound(Sprites[j].num,Match,jRect);
            if (Intersect(iRect,jRect)) then
            begin
                OverHideSprite(j,Match);
            end;
          end;
       end;
       HideSprite(Sprites[i]);
    end;
end;

procedure DrawWorld( Redraw:integer; var Match:typeMATCH_STATE);
var i:integer;
    x,y:integer;
begin
     if (Redraw = REDRAW_ALL) then
     begin
         ClearDevice;
         FnDrawBack;
         for i := 1 to nSprites do begin
             ResetSprite(Sprites[i]);
         end;
         for i := 1 to nTurnSprites do begin
             ResetSprite(TurnSprites[i]);
         end;
        for i := 1 to nSprites do begin
            DrawSprite(Sprites[i],Match);
        end;
        for i := 1 to nTurnSprites do begin
            DrawSprite(TurnSprites[i],Match);
        end;
     end
     else
     begin
{         if Settings.bWaitRetrace then
             VSinc;}
         for i := 1 to nSprites do begin
             OverHideSprite(i,Match);
             DrawSprite(Sprites[i],Match);
         end;
         if (Redraw = REDRAW_TURN) then
             for i := 1 to nTurnSprites do
             begin
                 HideSprite(TurnSprites[i]);
                 DrawSprite(TurnSprites[i],Match);
             end;
     end;
end;

begin
    nSprites := 0;
end.
